using HackathonPreparing.ApiService.Features.WeatherForecastFeature.EfCore;
using Microsoft.AspNetCore.Authorization;
using Microsoft.EntityFrameworkCore;

namespace HackathonPreparing.ApiService.Features.WeatherForecastFeature;

public static class WeatherForecastEndpointsExtensions
{
    public static void AddWeatherForecastEndpoints (this WebApplication app)
    {   
        var group = app.MapGroup("weatherforecast");
        group.MapGet(string.Empty, async (WeatherForecastContext db) => await db.Forecasts.ToListAsync());
        group.MapDelete("{id}", [Authorize(Roles="moderator")]async (WeatherForecastContext db, int id) =>
        {
            var forecast = await db.Forecasts.FindAsync(id);
            if (forecast == null)
            {
                return Results.NotFound();
            }

            db.Forecasts.Remove(forecast);
            await db.SaveChangesAsync();

            return Results.NoContent();
        });
        group.MapPost(string.Empty, [Authorize(Roles="moderator")] async (WeatherForecastContext db, WeatherForecast forecast) =>
        {
            db.Forecasts.Add(forecast);
            await db.SaveChangesAsync();

            return Results.Created($"/weatherforecast/{forecast.Id}", forecast);
        });
        group.MapPut("{id}", [Authorize(Roles="moderator")] async (WeatherForecastContext db, int id, WeatherForecast forecast) =>
        {
            if (id != forecast.Id)
            {
                return Results.BadRequest();
            }

            db.Entry(forecast).State = EntityState.Modified;

            try
            {
                await db.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (await db.Forecasts.FindAsync(id) == null)
                {
                    return Results.NotFound();
                }

                throw;
            }

            return Results.NoContent();
        });
        group.WithTags("weatherforecast");
    }
}